@@ -5,36 +5,37 @@ from sql.aggregate import Max
 from sql.conditionals import Coalesce
 from sql.functions import CurrentTimestamp, LastValue
 
-from trytond.model import ModelView, ModelSQL, fields
-from trytond.pool import Pool, PoolMeta
-from trytond.transaction import Transaction
-
+from trytond.model import ModelSQL, ModelView, convert_from, fields
 from trytond.modules.product import round_price
-
-
-def convert_from(table, tables):
-    right, condition = tables[None]
-    if table:
-        table = table.join(right, condition=condition)
-    else:
-        table = right
-    for k, sub_tables in tables.items():
-        if k is None:
-            continue
-        table = convert_from(table, sub_tables)
-    return table
+from trytond.pool import Pool, PoolMeta
+from trytond.tools import timezone as tz
+from trytond.transaction import Transaction
 
 
 class Product(metaclass=PoolMeta):
     __name__ = 'product.product'
 
     def get_multivalue(self, name, **pattern):
+        pool = Pool()
+        Company = pool.get('company.company')
         context = Transaction().context
         if (name == 'cost_price'
                 and context.get('_datetime')
                 and self.type in ['goods', 'assets']):
-            cost_price = self.get_cost_price_at(
-                context['_datetime'].date(), **pattern)
+            datetime = context['_datetime']
+            company = pattern.get(
+                'company', Transaction().context.get('company'))
+            if company:
+                company = Company(company)
+                if company.timezone:
+                    timezone = tz.ZoneInfo(company.timezone)
+                    try:
+                        datetime = (
+                            datetime.replace(tzinfo=tz.UTC).astimezone(
+                                timezone))
+                    except OverflowError:
+                        pass
+            cost_price = self.get_cost_price_at(datetime.date(), **pattern)
             if cost_price is not None:
                 return cost_price
         return super().get_multivalue(name, **pattern)
@@ -78,33 +79,38 @@ class ProductCostHistory(ModelSQL, ModelView):
         Product = pool.get('product.product')
         Template = pool.get('product.template')
         CostPrice = pool.get('product.cost_price')
+        User = pool.get('res.user')
         move = Move.__table__()
         product = Product.__table__()
         template = Template.__table__()
         history = CostPrice.__table_history__()
-        database = Transaction().database
+        transaction = Transaction()
+        database = transaction.database
+        user = User(transaction.user)
 
         tables, clause = Move.search_domain([
                 ('cost_price', '!=', None),
                 ('state', '=', 'done'),
-                Product()._domain_moves_cost(),
-                # Incoming moves
-                ('to_location.type', '=', 'storage'),
-                ('from_location.type', '!=', 'storage'),
+                Product._domain_moves_cost(),
+                ['OR',
+                    Product._domain_in_moves_cost(),
+                    Product._domain_out_moves_cost(),
+                    ],
                 ], tables={
                 None: (move, None),
                 })
 
+        cost_price = Coalesce(move.product_cost_price, move.cost_price)
         if database.has_window_functions():
             window = Window(
                 [move.effective_date, move.product],
                 frame='ROWS', start=None, end=None,
                 order_by=[move.write_date.asc, move.id.asc])
-            cost_price = LastValue(move.cost_price, window=window)
+            cost_price = LastValue(cost_price, window=window)
         else:
-            cost_price = cls.cost_price.sql_cast(move.cost_price)
+            cost_price = cls.cost_price.sql_cast(cost_price)
 
-        move_history = convert_from(None, tables).select(
+        move_history = convert_from(None, tables, type_='INNER').select(
             (move.id * 2).as_('id'),
             move.effective_date.as_('date'),
             move.product.as_('product'),
@@ -121,8 +127,12 @@ class ProductCostHistory(ModelSQL, ModelView):
             Max(move_history.cost_price).as_('cost_price'),
             group_by=[move_history.date, move_history.product])
 
+        if user.company:
+            timezone = user.company.timezone
+        else:
+            timezone = None
         price_datetime = Coalesce(history.write_date, history.create_date)
-        price_date = cls.date.sql_cast(price_datetime)
+        price_date = cls.date.sql_cast(price_datetime, timezone=timezone)
         if database.has_window_functions():
             window = Window(
                 [price_date, history.product],
@@ -141,7 +151,7 @@ class ProductCostHistory(ModelSQL, ModelView):
                 history.product.as_('product'),
                 cost_price.as_('cost_price'),
                 where=~template.type.in_(['goods', 'assets'])
-                & cls._non_moves_clause(history)))
+                & cls._non_moves_clause(history, user)))
 
         query |= price_history.select(
             Max(price_history.id).as_('id'),
@@ -156,8 +166,9 @@ class ProductCostHistory(ModelSQL, ModelView):
         return query
 
     @classmethod
-    def _non_moves_clause(cls, history_table):
-        return history_table.company == Transaction().context.get('company')
+    def _non_moves_clause(cls, history_table, user):
+        company_id = user.company.id if user.company else None
+        return history_table.company == company_id
 
     def get_rec_name(self, name):
         return str(self.date)
